from typing import Generator, Literal, List

from openai.types.chat.chat_completion import ChatCompletion, Choice, ChatCompletionMessage
from openai.types.chat.chat_completion_message_tool_call import ChatCompletionMessageToolCall

ChoiceFinishReason = Literal["stop", "length", "tool_calls", "content_filter", "function_call"]

class ChatCompletionHandler:
    def __init__(self, response: ChatCompletion):
        self.response: ChatCompletion = response

    @property
    def choice(self) -> Choice:
        return self.response.choices[0]

    @property
    def message(self) -> ChatCompletionMessage:
        return self.choice.message

    @property
    def finish_reason(self) -> ChoiceFinishReason:
        """ Razón por la que el modelo deja de generar tokens."""
        return self.choice.finish_reason

    @property
    def tool_calls(self) -> List[ChatCompletionMessageToolCall]:
        return self.message.tool_calls

    def iter_tool_call(self) -> Generator[ChatCompletionMessageToolCall, None, None]:
        return (tool_call for tool_call in self.tool_calls)



from typing import Generator, Tuple

from openai import Stream
from openai.types.chat.chat_completion_chunk import ChatCompletionChunk

CONTENT = "content"

T_IterItemsStream = Generator[ChatCompletionChunk, None, None]

def iter_blocks(*iterables: list, len_blocks: int = 100) -> Generator[Tuple[list], None, None]:
    """ Itera en un conjunto de iterables en bloques de a `len_blocks`."""
    for idx in range(0, len(iterables[0]), len_blocks):
        r = tuple(iterables[i][idx: idx + len_blocks] for i in range(len(iterables)))
        if len(r) == 1:
            r = r[0]
        yield r

def iter_answer_partial(stream: Stream) -> Generator[str, None, None]:
    iter_items: T_IterItemsStream = (item for item in stream)
    for item in iter_items:
        if hasattr(item.choices[0].delta, CONTENT):
            answer_partial = item.choices[0].delta.content
            if not answer_partial:
                continue
            yield answer_partial

def answer_from_stream(stream: Stream) -> str:
    answer = ""
    for answer_partial in iter_answer_partial(stream=stream):
        answer += answer_partial
    return answer



from typing import Literal, List
from pathlib import Path
from abc import ABC

from openai import OpenAI, Stream
from openai.resources.chat.completions import Completions
from openai.resources.embeddings import Embeddings
from openai.resources.audio.transcriptions import Transcriptions

from oaikit.utils import iter_blocks

#from oaikit.utils import answer_from_stream
#DEFAULT_OPENAI_API_BASE = "https://api.openai.com/v1"
#openai.api_base = os.getenv(OPENAI_API_BASE, DEFAULT_OPENAI_API_BASE)

MODEL_EMBEDDING = "text-embedding-ada-002"
DEFAULT_MODEL = "gpt-4o"
LANGUAGE_DEFAULT = "es"

class BaseOAI(ABC):
    def __init__(self, *, api_key: str):
        self._client = OpenAI(api_key=api_key)

    @property
    def client(self) -> OpenAI:
        return self._client
    
    @property
    def completions(self) -> Completions:
        return self.client.chat.completions

    @property
    def embeddings(self) -> Embeddings:
        return self.client.embeddings

    @property
    def transcriptions(self) -> Transcriptions:
        return self.client.audio.transcriptions
    
    def transcript(
            self,
            *,
            path_file: Path,
            language: Literal["es", "en"] = LANGUAGE_DEFAULT,
            model: str = "whisper-1"
        ) -> str:
        transcription = self.transcriptions.create(
            model = model,
            file = open(path_file, "rb"),
            language = language
        )
        return transcription.text

    def get_vectors(self, *, inputs: List[str], model = MODEL_EMBEDDING, len_blocks = 100) -> List[List[float]]:
        """ Retorna los vectores de embeddings para una lista de strings."""
        assert isinstance(inputs, list) and all(isinstance(i, str) for i in inputs), "Se espera una lista de strings a convertir en vectores."
        vectors = []
        for block in iter_blocks(inputs, len_blocks=len_blocks):
            embeddings = self.embeddings.create(input=block, model=model).data
            vectors.extend([e.embedding for e in embeddings])
        return vectors

    # def get_stream(self, *, messages: List[Dict[str, str]], model: str) -> Stream:
    #     return self.completions.create(model=model, messages=messages, stream=True)

    # def answer_from_messages(self, *, messages: List[Dict[str, str]], model: str = DEFAULT_MODEL) -> str:
    #     stream = self.get_stream(messages=messages, model=model)
    #     answer = answer_from_stream(stream=stream)
    #     return answer


class OAI(BaseOAI):
    def __init__(self, *, api_key: str):
        super().__init__(api_key=api_key)



from typing import Literal, List, Any, Tuple
from abc import ABC, abstractmethod

from pydantic import BaseModel

__all__ = ["FuncPropStr", "FuncPropEnum", "PropsBase", "DataPropsBase"]

TYPE_STRING = "string"
T_String = Literal["string"]

class FuncPropStr(BaseModel):
    type: T_String = TYPE_STRING
    description: str

class FuncPropEnum(BaseModel):
    type: T_String = TYPE_STRING
    enum: List[str]     # Categorical values.
    description: str


class PropsBase(BaseModel, ABC):
    @classmethod
    @abstractmethod
    def fn_name(cls) -> str:
        """ Retorna el nombre de la function call asociada."""
        ...


class DataPropsBase(BaseModel, ABC):
    @classmethod
    @abstractmethod
    def fn_name(cls) -> str:
        """ Retorna el nombre de la function call asociada."""
        ...
    
    @abstractmethod
    def execute(self, *args, **kwargs) -> Tuple[str, Any]:
        """ Ejecuta la función y realiza acciones."""
        ...
